package at.ac.uibk.gitsearch.service;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.io.IOException;
import java.time.Instant;
import java.time.LocalDateTime;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;

import javax.mail.internet.MimeMessage;

import org.codeability.sharing.plugins.api.search.SearchResultDTO;
import org.elasticsearch.node.NodeValidationException;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.Spy;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.MessageSource;
import org.springframework.data.auditing.DateTimeProvider;
import org.springframework.mail.javamail.JavaMailSenderImpl;
import org.springframework.transaction.annotation.Transactional;
import org.thymeleaf.spring5.SpringTemplateEngine;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.domain.UserWatchList;
import at.ac.uibk.gitsearch.domain.WatchListEntry;
import at.ac.uibk.gitsearch.domain.enumeration.CheckFrequency;
import at.ac.uibk.gitsearch.repository.UserRepository;
import at.ac.uibk.gitsearch.repository.UserWatchListRepository;
import at.ac.uibk.gitsearch.repository.search.MetaDataRepository;
import at.ac.uibk.gitsearch.repository.search.testESService.ElasticSearchTestServerConfiguration;
import at.ac.uibk.gitsearch.service.dto.UserDTO;
import io.github.jhipster.config.JHipsterProperties;

/**
 * Integration tests for {@link UserWatchListService}.
 */
@SpringBootTest(classes = GitsearchApp.class)
@Transactional
public class InfoMailServiceIT {

    private static final String DEFAULT_LOGIN = "johndoe";

    private static final String DEFAULT_EMAIL = "johndoe@localhost";

    private static final String DEFAULT_FIRSTNAME = "john";

    private static final String DEFAULT_LASTNAME = "doe";

    private static final String DEFAULT_IMAGEURL = "http://placehold.it/50x50";

    private static final String DEFAULT_LANGKEY = "dummy";
    
    private static final String TEST_EXERCISE_ID = "3";


    @Autowired
    private JHipsterProperties jHipsterProperties;

    @Autowired
    private MessageSource messageSource;

    @Autowired
    private SpringTemplateEngine templateEngine;

    @Autowired
    private UserRepository userRepository;
    @Autowired
    private UserService userService;

    @Autowired
    private UserWatchListService userWatchListService;
    @Autowired
    private UserWatchListRepository userWatchListRepository;
    @Autowired
    private MetaDataRepository metaDataRepository;

    @Spy
    private JavaMailSenderImpl javaMailSender;

    @Captor
    private ArgumentCaptor<MimeMessage> messageCaptor;

    private MailService mailService;
    @Mock
    private DateTimeProvider dateTimeProvider;

    private User user;

	@BeforeAll
	public static void setUpESServer() throws IOException, NodeValidationException {
		ElasticSearchTestServerConfiguration.getTestInstance().startTestNode();
	}


    @BeforeEach
    public void init() {
        UserDTO userDTO = new UserDTO();
        userDTO.setLogin(DEFAULT_LOGIN);

        userDTO.setActivated(true);
        userDTO.setEmail(DEFAULT_EMAIL);
        userDTO.setFirstName(DEFAULT_FIRSTNAME);
        userDTO.setLastName(DEFAULT_LASTNAME);
        userDTO.setImageUrl(DEFAULT_IMAGEURL);
        userDTO.setLangKey(DEFAULT_LANGKEY);
        userDTO.setAuthorities(new HashSet<>());
        userDTO.getAuthorities().add("sharing");

        when(dateTimeProvider.getNow()).thenReturn(Optional.of(LocalDateTime.now()));
        
        userDTO.setActivated(false);
        user = userService.createUser(userDTO);
        
        UserWatchList wlDAILY = new UserWatchList("someTestWatchList", CheckFrequency.DAILY, user);

        WatchListEntry wle1 = new WatchListEntry();
		wle1.setExerciseId(TEST_EXERCISE_ID);
        wle1.setExerciseName("Unused in this test");
        wlDAILY.getWatchListEntries().add(wle1);
        
        userWatchListRepository.save(wlDAILY);
        
        UserWatchList wlMONTHLY = new UserWatchList("someTestWatchList", CheckFrequency.MONTHLY, user);

        WatchListEntry wle2 = new WatchListEntry();
        wle2.setExerciseId(TEST_EXERCISE_ID);
        wle2.setExerciseName("Unused in this test");
        wlMONTHLY.getWatchListEntries().add(wle2);
        
        userWatchListRepository.save(wlMONTHLY);

    }
    
    
    @BeforeEach
    public void setup() {
        MockitoAnnotations.initMocks(this);
        doNothing().when(javaMailSender).send(any(MimeMessage.class));
        mailService = new MailService(jHipsterProperties, javaMailSender, messageSource, templateEngine, userRepository, userWatchListService);
    }

    
    @Test
    @Transactional
    public void testSendInfoMail() throws Exception {
// 	not working
//        when(dateTimeProvider.getNow()).thenReturn(Optional.of(LocalDateTime.ofEpochSecond(LAST_CHANGE_DATE_OF_EXERCISE3.getEpochSecond(), 0, ZoneOffset.UTC)));
    	final SearchResultDTO testExercise = metaDataRepository.getExerciseById(TEST_EXERCISE_ID);
    	testExercise.getProject().setLast_activity_at(Instant.now());
    	
    	// this test data contradicts with unique exercise id
    	ElasticSearchTestServerConfiguration.getTestInstance().updateMetadata(testExercise, 3);
    	
        mailService.sendInfoMails();
        verify(javaMailSender).send(messageCaptor.capture());
        List<MimeMessage> messages = messageCaptor.getAllValues();
        assertThat(messages.size()).isEqualTo(1);
        
        MimeMessage message = messages.get(0);
        assertThat(message.getAllRecipients()[0].toString()).isEqualTo(user.getEmail());
        assertThat(message.getFrom()[0].toString()).isEqualTo(jHipsterProperties.getMail().getFrom());
        assertThat(message.getContent().toString()).isNotEmpty();
        assertThat(message.getDataHandler().getContentType()).isEqualTo("text/html;charset=UTF-8");
    }



 }
